"""

  RTF Export example
  
  Copyright 2006 Mikko Ohtamaa
  
  This example is not fully working, but it should show you 
  guidelines how to integrate RTF Export to your own products.

"""

# Python imports
import StringIO

# Zope imports
from AccessControl import getSecurityManager, ClassSecurityInfo, Unauthorized

# Plone imports
from Products.Archetypes.public import *
from Products.Archetypes.ExtensibleMetadata import ExtensibleMetadata
from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.base import updateActions, updateAliases
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.CMFCorePermissions import View, ManageProperties, ListFolderContents, ModifyPortalContent

# local imports
from Permissions import * # View permission
from config import PROJECTNAME
from Products.RTFExport.RTFGenerator import RTFGenerator
from Products.RTFExport.PyRTF import *

#
# Some example stuff
#
schema = ATContentTypeSchema.copy() + Schema(
    # field list
    (
     
        DateTimeField("birthday",
            searchable=True,
            required=False,
            widget=CalendarWidget(
                label="Born",
                description="The time of birth",
                format="%d.%m.%Y",
                show_hm=False,
                ),
            ),

         TextField('biography',
            searchable=False,
            required=False,
            widget=TextAreaWidget(
                ),
           ),    
               
         TextField('bloopers',
            searchable=True,
            required=False,
            widget=TextAreaWidget(
                ),
           ),         
    )
    #other schema items (none)
) 

# Modifications to base schema
class Exampe(ATCTContent):
    """ Example class how to use RTF export
    
    
    """

    schema = schema

    global_allow = True
        
    # Generate user friendly id from title during creation
    _at_rename_from_title = True
    
    typeDescription= 'RTF Export Example'
    meta_type      = 'RTFExportExample'
    archetype_name = 'RTFExportExample'
    
    # Security manager object for this class
    security = ClassSecurityInfo()
        
    # Add RTF export to your type document actions
    actions = updateActions(ATCTContent,
        (
        {
        'id'          : 'export_rtf',
        'name'        : 'Export as RTF',
        'action'      : 'string:$object_url/export_rtf',
        'permissions' : (View,),
        'category'    : "document_actions"
        },
        )
        )
  
    def generateRTF(self):
        """ Get item contents as a RTF document
        
        If you define export_rtf action you must also define
        generateRTF method for your class. 
        
        This method is called by export_rtf.py in skins folder.
       
        @return string containing RTF data
        """
        gen = EmployeeRTFGenerator(self)
        return gen.generateRTF()
                            

registerType(Employee, PROJECTNAME)

class EmployeeRTFGenerator(RTFGenerator):
    """ Employee RTF exporter """
    
    def getExportableFields(self):
        """ Defines exported fields and field order
        
        This method allows you to filter/reorder fields for RTF documents.   
        
        Return list of field names to be exported in RTFGenerator.exportFields     
        """
        
        # Do not export bloopers field
        return [
            "born",
            "biography"
        ]
        
    def fillContents(self):
        """ Generate RTF document using PyRTF 
        
        - Create document header and footer
        - Call RTFGenerator.exportFields to automatically export fields
        
        This method is called by RTFGenerator.generateRTF which
        later on transforms PyRTF elements to RTF code.
        
        """
        
        # variable shortcuts (no need to type self.item)
        item = self.item
        section = self.section
        ss = self.doc.StyleSheet
        
        # Document title is Curriculum Vitae + Employee name
        p = Paragraph( ss.ParagraphStyles.Heading1 )
        p.append('Sample Record - ' + self.item.Title())
        section.append(p)
        
        # Embed modification timestamp
        p = Paragraph( ss.ParagraphStyles.Normal)
        p.append('Record  updated ' + self.item.modified().strftime("%d.%m.%Y"))
        section.append(p)
        
        # Export Archetypes fields defined in getExportableFields()
        self.exportFields()

        